// BasicComponent.h: interface for the BasicComponent class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_BASICCOMPONENT_H__2768E01E_6025_44A6_9033_83B6E363BE80__INCLUDED_)
#define AFX_BASICCOMPONENT_H__2768E01E_6025_44A6_9033_83B6E363BE80__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Component.h"
#include "Identifier.h"

#include <deque>

class ComponentVariant;
class ComponentRegister;

class BasicComponent
{
public:
	void addVariant(ComponentVariant * var);

	void getMinimumSize(long & xMin, long & xMax, long & yMin, long & yMax);
	void getMaximumSize(long & xMin, long & xMax, long & yMin, long & yMax);
	void getMinimumSize(Direction dir, long & xMin, long & xMax, long & yMin, long & yMax);
	void getMaximumSize(Direction dir, long & xMin, long & xMax, long & yMin, long & yMax);
	inline void getMinimumSize(long dx, long dy, Direction dir, long & xMin, long & xMax, long & yMin, long & yMax)
	{
		xMin = m_minLoX[dir] + dx;
		xMax = m_minHiX[dir] + dx;
		yMin = m_minLoY[dir] + dy;
		yMax = m_minHiY[dir] + dy;
	}

	void deactivateVariant(ComponentVariant * var);
	BasicComponent(
		ComponentRegister * reg,
		long id,
		const char * name,
		Direction dir,
		long dx,
		long dy,
		long dt);

	virtual ~BasicComponent();


	void print();

	inline const char * getName() { return m_name; };

	inline long getId() { return m_id; };

	void prepare(long period, long compression);

	inline getActive() { return m_variantListLength > 0; };
	inline long getNumVariants() { return m_variantListLength; };
	inline long getQueueSize() { return m_variantQueue.size(); };
	inline ComponentRegister * getRegister() { return m_register; };
	inline ComponentVariant * getActiveVariant(long i) { return m_variantList[i]; };
	inline Direction getDirection() { return m_dir[TF]; };
	inline Direction getDirection(Direction dir) { return m_dir[dir]; };
	inline long getDt() { return m_dt; };
	inline long getDx() { return m_dx[TF]; };
	inline long getDx(Direction dir) { return m_dx[dir]; };
	inline long getDy() { return m_dy[TF]; };
	inline long getDy(Direction dir) { return m_dy[dir]; };
	inline long getRdx() { return m_rdx[TF]; };
	inline long getRdx(Direction dir) { return m_rdx[dir]; };
	inline long getRdy() { return m_rdy[TF]; };
	inline long getRdy(Direction dir) { return m_rdy[dir]; };
	inline Direction getRdir() { return m_rDir[TF]; };
	inline Direction getRdir(Direction dir) { return m_rDir[dir]; };

private:
	void deleteVariantList();

	ComponentRegister * m_register;
	long m_id;

	long m_dt;
	long m_dx[8];
	long m_dy[8];
	Direction m_dir[8];

	long m_rdx[8];
	long m_rdy[8];
	Direction m_rDir[8];

	char * m_name;

	typedef std::deque<ComponentVariant *> VariantQueue;
	typedef VariantQueue::iterator VariantIterator;

	VariantQueue m_variantQueue;

	ComponentVariant ** m_variantList;
	long m_variantListLength;

	long m_minLoX[8];
	long m_minHiX[8];
	long m_minLoY[8];
	long m_minHiY[8];

	long m_maxLoX[8];
	long m_maxHiX[8];
	long m_maxLoY[8];
	long m_maxHiY[8];

protected:
};

#endif // !defined(AFX_BASICCOMPONENT_H__2768E01E_6025_44A6_9033_83B6E363BE80__INCLUDED_)

