// PointSet.h: interface for the PointSet class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_POINTSET_H__F21502AA_0B02_4EEE_AC69_12D628068908__INCLUDED_)
#define AFX_POINTSET_H__F21502AA_0B02_4EEE_AC69_12D628068908__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Identifier.h"

class PointSet  
{
public:
	long getCount();
	bool deleteAndNext(long & x, long & y, Direction & dir);
	bool getNext(long & x, long & y, Direction & dir);
	bool getFirst(long & x, long & y, Direction & dir);
	bool checkPoint(long x, long y, Direction dir);
	void addPoint(long x, long y, Direction dir);
	PointSet();
	virtual ~PointSet();

	enum
	{
		c05k = 512 / sizeof(unsigned short),
		c1k = 1024 / sizeof(unsigned short),
		c2k = 2048 / sizeof(unsigned short),
		c4k = 4096 / sizeof(unsigned short),
		c8k = 8192 / sizeof(unsigned long)
	};

	static void printStatistics();

protected:

	union Block05k
	{
		Block05k * next;
		unsigned short data[c05k];
	};

	union Block1k
	{
		Block1k * next;
		Block05k blk[2];
		unsigned short data[c1k];
	};

	union Block2k
	{
		Block2k * next;
		Block1k blk[2];
		unsigned short data[c2k];
	};

	union Block4k
	{
		Block4k * next;
		Block2k blk[2];
		unsigned short data[c4k];
	};

	union Block8k
	{
		Block8k * next;
		Block4k blk[2];
		unsigned long map[c8k]; // used as bitmap
	};

	union BlockPtr
	{
		Block8k * p8k;
		Block4k * p4k;
		Block2k * p2k;
		Block1k * p1k;
		Block05k * p05k;
	};

	struct MemoryBlock // allocation unit
	{
		Block8k blk[128];
	};

	struct MapHeader
	{
		long count;
		long maxCount;
		// the pointer type depends on the count
		BlockPtr pointer;
	};

	void makePassXYD();
	bool getFirstDetail(MapHeader * mh, unsigned short & v);
	bool getNextDetail(MapHeader * mh, unsigned short & v);
	bool deleteAndNextDetail(MapHeader * mh, unsigned short & v);

private:

	static Block8k * getBlock8k();
	static Block4k * getBlock4k();
	static Block2k * getBlock2k();
	static Block1k * getBlock1k();
	static Block05k * getBlock05k();

	static void putBlock8k(Block8k * b);
	static void putBlock4k(Block4k * b);
	static void putBlock2k(Block2k * b);
	static void putBlock1k(Block1k * b);
	static void putBlock05k(Block05k * b);

	static Block8k * s_free8k;
	static Block4k * s_free4k;
	static Block2k * s_free2k;
	static Block1k * s_free1k;
	static Block05k * s_free05k;

	static long stat_used8k;
	static long stat_used4k;
	static long stat_used2k;
	static long stat_used1k;
	static long stat_used05k;

	static long stat_free8k;
	static long stat_free4k;
	static long stat_free2k;
	static long stat_free1k;
	static long stat_free05k;

private:

	MapHeader m_index[200];
	long m_passNdx;
	long m_passX;
	long m_passY;
	Direction m_passDir;

	long m_passDetailNdx;
	unsigned long m_passDetailBit;
	long m_passDetailBitNo;
};

#endif // !defined(AFX_POINTSET_H__F21502AA_0B02_4EEE_AC69_12D628068908__INCLUDED_)
