// Search.h: interface for the Search class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SEARCH_H__8EE30F9D_C0F5_468B_8D31_22ED7D2736CB__INCLUDED_)
#define AFX_SEARCH_H__8EE30F9D_C0F5_468B_8D31_22ED7D2736CB__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#pragma warning(disable: 4786)
#endif // _MSC_VER > 1000

#include <deque>
#include <queue>
#include <set>
#include <functional>
#include <vector>

#include "Component.h"
#include "CompoundComponent.h"

class BasicComponent;
class ComponentRegister;
class AcTreeNode;
class PointSet;

class Search  
{
public:
	Search();
	virtual ~Search();

	void addTarget(long dx1, long dy1, long dt1, Direction dir1, long dx2, long dy2, long dt2, Direction dir2);
	void addTarget(long dx, long dy, long dt, Direction dir, Direction startDir);
	void addTarget(long dx, long dy, long dt, Direction dir);

	void trimTargets(long maxDt);

    void test();
	void run(
		long period, 
		long compression,
		long maxCells,
		long maxResults,
		bool sorted,
		long treeSizeKB,
		long ignoreSets,
		bool useMemory);
	void runCheapWay();
	void runExtensiveWay();
    void interrupt();

private:
	struct Target
	{
		Target(long x, long y, long t, Direction d, Direction s)
		{
			dx = x;
			dy = y;
			dt = t;
			dir = d;
			sDir = s;
			closed = (x == 0) && (y == 0) && (dir == TF);
		}

		long dx;
		long dy;
		long dt;
		Direction dir;
		Direction sDir;
		bool closed;
	};

	struct StackItem  // stack item for the 'extensive search' algorithm
	{
		// A/C Tree position for this stack depth
		AcTreeNode * nd;
		// local component index
		long compIdx;
		// position in time (starts at target dt and goes to zero)
		long dt;
		// result minimum size
		long x1, x2, y1, y2;
	};

	bool checkAcTree(AcTreeNode * nd);
	ComponentRegister * m_register;
	long m_numSets;
	bool m_interrupted;
	long m_maxResults;
	bool m_run;
	void printComponent(CompoundComponent * comp, long setNo);
	void printSorted();
	void printResult(CompoundComponent * comp, Direction dir);

	void permute(long * activeSet, Target * target);
	BasicComponent ** m_componentList;
	long m_componentListLength;

	void searchTarget(Target * target, bool solve); // the 'cheap' way
	void searchTarget(Target * target, PointSet * pointSet, bool solve); // the 'expensive' way
	void continueSearch(PointSet * pointSet, StackItem * stack, long maxComponentIdx, bool closed, bool optimize, Target * target);
	bool isResultSuitable(CompoundComponent * cc);

	long m_maxCells;
	bool m_optimizeMaxCells;
	bool m_sorted;
	bool m_closed;
	long m_ignoreSets;

	struct CompSet
	{
		long num;
		BasicComponent * comp;
		long length;
	};

	void scanPermutations(CompSet * best, Target * target);

	CompSet * m_set;
	long m_setSize;
	long m_setCount;
	long m_setSumLen;
	CompoundComponent * m_cc;

    struct ccless : public std::less<CompoundComponent *>
	{
		bool operator()(CompoundComponent * x, CompoundComponent * y) const 
		{ 
			return x->getArea() > y->getArea();
		}
    };

	typedef std::priority_queue<CompoundComponent *, std::vector<CompoundComponent *>, ccless> ResultList;
	ResultList m_resultList;

	struct tgless : public std::less<Target *>
	{
		bool operator()(Target * x, Target * y) const
		{
			if (x->dt < y->dt)
			{
				return true;
			}
			else if (x->dt == y->dt)
			{
				if (x->dx < y->dx)
				{
					return true;
				}
				else if (x->dx == y->dx)
				{
					if (x->dy < y->dy)
					{
						return true;
					}
					else if (x->dy == y->dy)
					{
						return x->dir < y->dir;
					}
				}
			}
			return false;
		}
	};

	typedef std::set<Target *, tgless> TargetList;

	TargetList m_targetList;

};

#endif // !defined(AFX_SEARCH_H__8EE30F9D_C0F5_468B_8D31_22ED7D2736CB__INCLUDED_)
